﻿using System;
using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Runtime.InteropServices;


namespace Nerv36
{
    public enum Sync_model : Byte
    {
        on_timer = 1,
        on_post = 2,
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public struct Address
    {
        public UInt32 ip;
        public UInt16 port;
        [MarshalAs(UnmanagedType.LPStr)] public string id;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct Message_address
    {
        public UInt16 port;
        public UInt16 code;
        public Byte transaction;
    }
    [StructLayout(LayoutKind.Sequential, Pack=1)]
    public struct Variable_array_field_dctrl
    {
        public UInt32 dindex;
        public UInt32 dsize;
    }

    public static class Util
    {
        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_get_err_str")]
        public static extern string get_err_str( Int32 _err);

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_get_address")]
        public static extern UIntPtr get_address( string _name_, ref Address _stub_addr_);

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_inet_addr")]
        public static extern UInt32 inet_addr(string _ip_);

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_inet_ntoa")]
        private static extern IntPtr nerv_inet_ntoa(UInt32 _ip);

        public static string inet_ntoa(UInt32 _ip)
        {
            IntPtr ptr = nerv_inet_ntoa(_ip);
            return Marshal.PtrToStringAnsi(ptr);
        }

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_initialize_default")]
        public static extern void initialize_default();

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_initialize")]
        public static extern void initialize(UIntPtr _thread_count, UIntPtr _call_buffer_count, UIntPtr _udp_buffer_count);

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_finalize")]
        public static extern void finalize();

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_msg_processing_by_serial_group")]
        public static extern void msg_processing_by_serial_group(Message_address _a, Message_address _b);

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_msg_processing_by_serial_alone")]
        public static extern void msg_processing_by_serial_alone(Message_address _a);

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_msg_processing_by_parallel")]
        public static extern void msg_processing_by_parallel(Message_address _a);

    }


    public class Output
    {
        public static void Log(string value)
        {
#if UNITY_64
            UnityEngine.Debug.Log(value);
#else
            Console.WriteLine(value);
#endif
        }
    }
}
