using System;
using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace Nerv36
{
    public interface IStub
    {
        void on_connect(Address _proxy_addr);
        void on_disconnect(Address _proxy_addr);
    }

    public class Stub
    {
        // nerv_stub delegate
        public delegate void on_connect_ft(IntPtr _object_, Address _proxy_addr);
        public delegate void on_disconnect_ft(IntPtr _object_, Address _proxy_addr);

        static on_connect_ft on_connect_ = new on_connect_ft(on_connect);
        static on_disconnect_ft on_disconnect_ = new on_disconnect_ft(on_disconnect);

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_stub_alloc")]
        public static extern IntPtr alloc(on_connect_ft _on_connect_, on_disconnect_ft _on_disconnect_);

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_stub_free")]
        public static unsafe extern void free(IntPtr _object_);

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_stub_start")]
        public static extern Int32 start(IntPtr _object_, string _name_, ref UInt16 _port_);

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_stub_stop")]
        public static extern Int32 stop(IntPtr _object_);

        [DllImport("64nerv36.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_stub_is_started")]
        public static extern Boolean is_started(IntPtr _object_);

        static ConcurrentDictionary<IntPtr, Stub> dict = new ConcurrentDictionary<IntPtr, Stub>();

        public IntPtr object_;
        public IStub istub;

        public static void on_connect(IntPtr _object_, Address _proxy_addr)
        {
            // nerv_stub object = ;
            dict[_object_].istub.on_connect(_proxy_addr);
        }

        public static void on_disconnect(IntPtr _object_, Address _proxy_addr)
        {
            dict[_object_].istub.on_disconnect(_proxy_addr);
        }

        public Stub(IStub _istub)
        {
            object_ = alloc(on_connect_, on_disconnect_);
            dict[object_] = this;
            istub = _istub;
        }

        ~Stub()
        {
            // API.nerv_stub_free(inst_);
            // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
            // C#에서는 API.nerv_stub_free(inst_); 로 정리하지 않는다.
            // C#의 소멸자는 언제 호출될지 알수 없다.

            Stub obj;
            dict.TryRemove(object_, out obj);
        }

        public Int32 start(string _name_, ref UInt16 _port_)
        {
            return start(object_, _name_, ref _port_);
        }

        public Int32 stop()
        {
            return stop(object_);
        }

        public Boolean is_started()
        {
            return is_started(object_);
        }
    }
}
