<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" omit-xml-declaration="yes" indent="no"/>
<xsl:param name = "export_data_domain_path" />
<xsl:param name = "export_data_name" />
<xsl:template match="/">
  <xsl:for-each select="nerv_icde/datadomain/row">
    <xsl:if test="path=$export_data_domain_path">
      <xsl:variable name="open_namespace_path">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="' { namespace '" />
        </xsl:call-template></xsl:variable>
      <xsl:variable name="close_namespace_path">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="' */ } /* '" />
        </xsl:call-template>
      </xsl:variable><xsl:text/>
/******************************************************************************
    domain   : <xsl:value-of select="name"/>
    domain creator   : <xsl:value-of select="author"/>
    domain create date : <xsl:value-of select="create-day"/>
    domain description : <xsl:value-of select="description"/>
      <xsl:for-each select="data/row">
        <xsl:if test="name=$export_data_name">
    data name : <xsl:value-of select="name"/>
    data type : <xsl:value-of select="type"/>
    data writer   : <xsl:value-of select="../../author"/>
    data description: <xsl:value-of select="description"/>
*******************************************************************************/<xsl:text/>
using System;
using System.Runtime.InteropServices;
using Nerv36;

namespace Nerv36 { namespace Data <xsl:value-of select="$open_namespace_path" /> {<xsl:text>
</xsl:text>
          <xsl:variable name="value_type">
            <xsl:if test="type='char_t'">Byte</xsl:if>
            <xsl:if test="type='int64_t'">Int64</xsl:if>
            <xsl:if test="type='int32_t'">Int32</xsl:if>
            <xsl:if test="type='int16_t'">Int16</xsl:if>
            <xsl:if test="type='int8_t'">SByte</xsl:if>
            <xsl:if test="type='u_int64_t'">UInt64</xsl:if>
            <xsl:if test="type='u_int32_t'">UInt32</xsl:if>
            <xsl:if test="type='u_int16_t'">UInt16</xsl:if>
            <xsl:if test="type='u_int8_t'">Byte</xsl:if>
            <xsl:if test="type='ieee754_64_t'">Double</xsl:if>
            <xsl:if test="type='ieee754_32_t'">Single</xsl:if>
            <xsl:if test="type='scaled64_t'">Int64</xsl:if>
            <xsl:if test="type='scaled32_t'">Int32</xsl:if>
            <xsl:if test="type='scaled16_t'">Int16</xsl:if>
            <xsl:if test="type='u_scaled8_t'">SByte</xsl:if>
            <xsl:if test="type='u_scaled64_t'">UInt64</xsl:if>
            <xsl:if test="type='u_scaled32_t'">UInt32</xsl:if>
            <xsl:if test="type='u_scaled16_t'">UInt16</xsl:if>
            <xsl:if test="type='u_scaled8_t'">Byte</xsl:if>
            <xsl:if test="type='enum64_t'">UInt64</xsl:if>
            <xsl:if test="type='enum32_t'">UInt32</xsl:if>
            <xsl:if test="type='enum16_t'">UInt16</xsl:if>
            <xsl:if test="type='enum8_t'">Byte</xsl:if>
            <xsl:if test="type='alias'">
              <xsl:variable name="real_data_domain_namespace">
                <xsl:call-template name="get_data_domain_namespace_dot">
                <xsl:with-param name="data_domain_id" select="real_domain_id" />
                </xsl:call-template>
              </xsl:variable>Nerv36.Data<xsl:value-of select="$real_data_domain_namespace"/>.<xsl:value-of select="real_data"/>
            </xsl:if>
          </xsl:variable>

          <xsl:if test="type='enum64_t' or type='enum32_t' or type='enum16_t' or type='enum8_t'">
public enum <xsl:value-of select="name"/> : <xsl:value-of select="$value_type"/>
{
            <xsl:for-each select="datafield/row">
    <xsl:value-of select="member_name"/> = <xsl:value-of select="scmin"/>,
            </xsl:for-each>
}
          </xsl:if>
          <xsl:if test="type!='enum64_t' and type!='enum32_t' and type!='enum16_t' and type!='enum8_t'">
[StructLayout(LayoutKind.Sequential, Pack=1)]
public unsafe struct <xsl:value-of select="name"/>
{
            <xsl:if test="type='scaled64_t' or type='scaled32_t' or type='scaled16_t' or type='scaled8_t' or type='u_scaled64_t' or type='u_scaled32_t' or type='u_scaled16_t' or type='u_scaled8_t'">
    public const double min = <xsl:value-of select="scmin"/>;
    public const double max = <xsl:value-of select="scmax"/>;
            </xsl:if>
            <xsl:if test="type!='struct'">
              <xsl:if test="type!='alias'">
    public <xsl:value-of select="$value_type"/> value_buffer;

    public static implicit operator <xsl:value-of select="name"/>(<xsl:value-of select="$value_type"/> _obj)
    {
        <xsl:value-of select="name"/> temp;
        temp.value_buffer = _obj;
        return temp;
    }
    public static implicit operator <xsl:value-of select="$value_type"/>(<xsl:value-of select="name"/> p)
    {
        return p.value_buffer;
    }

              </xsl:if>
              <xsl:if test="type='alias'">
    fixed Byte value_buffer[(int)<xsl:value-of select="$value_type"/>._sizeof];


    public static ref <xsl:value-of select="name"/> malloc(Int32 _size = 65536)
    {
        fixed (<xsl:value-of select="$value_type"/>* p = &amp;<xsl:value-of select="$value_type"/>.malloc(_size))
        {
            return ref *(<xsl:value-of select="name"/>*)p;
        }
    }

    public ref <xsl:value-of select="name"/> realloc(UInt32 _size = 0)
    {
        ref <xsl:value-of select="$value_type"/> ori_ = ref ori();
        fixed (<xsl:value-of select="$value_type"/>* realloc_ = &amp;ori_.realloc() )
        {
            return ref *(<xsl:value-of select="name"/>*)realloc_;
        }
    }

    public void free()
    {
        ori().free();
    }

    public Int32 sizeofall()
    {
        return ori().sizeofall();
    }

    public void init_dynamic_buffer(Int64 _size)
    {
        ori().init_dynamic_buffer(_size);
    }

    public static implicit operator <xsl:value-of select="name"/>(<xsl:value-of select="$value_type"/> _obj)
    {
        <xsl:value-of select="name"/> temp;
        *(<xsl:value-of select="$value_type"/>*)temp.value_buffer = _obj;
        return temp;
    }
    public static implicit operator <xsl:value-of select="$value_type"/>(<xsl:value-of select="name"/> p)
    {
        return *(<xsl:value-of select="$value_type"/>*)p.value_buffer;
    }
    public ref <xsl:value-of select="$value_type"/> ori()
    {
        fixed ( byte* p = value_buffer)
        {
            return ref *(<xsl:value-of select="$value_type"/>*)p;
        }
    }
              </xsl:if>
    public UInt32 fit_size() { return ori().fit_size(); }
              <xsl:if test="type='alias'">
    public const Int32 _sizeof = <xsl:value-of select="$value_type"/>._sizeof;
              </xsl:if>
              <xsl:if test="type!='alias'">
    public const Int32 _sizeof = sizeof(<xsl:value-of select="$value_type"/>);
              </xsl:if>
            </xsl:if>
            <xsl:if test="type='struct'">
              <xsl:variable name="maxDynamicSize">
                <xsl:for-each select="datafield/row">
                  <xsl:variable name="original_type">
                    <xsl:if test="type='alias'">
                      <xsl:call-template name="get_original_type">
                        <xsl:with-param name="data_domain_id" select="real_domain_id" />
                        <xsl:with-param name="data" select="real_data" />
                      </xsl:call-template>
                    </xsl:if>
                    <xsl:if test="type!='alias'">
                      <xsl:value-of select="type"/>
                    </xsl:if>
                  </xsl:variable>
                  <xsl:variable name="real_data_domain_namespace">
                    <xsl:call-template name="get_data_domain_namespace_dot">
                      <xsl:with-param name="data_domain_id" select="real_domain_id" />
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:variable name="member_size">
                    <xsl:choose>
                      <xsl:when test="$original_type='char_t'">sizeof(Byte)</xsl:when>
                      <xsl:when test="$original_type='u_scaled64_t'">sizeof(UInt64)</xsl:when>
                      <xsl:when test="$original_type='u_scaled32_t'">sizeof(UInt32)</xsl:when>
                      <xsl:when test="$original_type='u_scaled16_t'">sizeof(UInt16)</xsl:when>
                      <xsl:when test="$original_type='u_scaled8_t'">sizeof(Byte)</xsl:when>
                      <xsl:when test="$original_type='scaled64_t'">sizeof(Int64)</xsl:when>
                      <xsl:when test="$original_type='scaled32_t'">sizeof(Int32)</xsl:when>
                      <xsl:when test="$original_type='scaled16_t'">sizeof(Int16)</xsl:when>
                      <xsl:when test="$original_type='scaled8_t'">sizeof(SByte)</xsl:when>
                      <xsl:when test="$original_type='int8_t'">sizeof(SByte)</xsl:when>
                      <xsl:when test="$original_type='int16_t'">sizeof(Int16)</xsl:when>
                      <xsl:when test="$original_type='int32_t'">sizeof(Int32)</xsl:when>
                      <xsl:when test="$original_type='int64_t'">sizeof(Int64)</xsl:when>
                      <xsl:when test="$original_type='u_int8_t'">sizeof(Byte)</xsl:when>
                      <xsl:when test="$original_type='u_int16_t'">sizeof(UInt16)</xsl:when>
                      <xsl:when test="$original_type='u_int32_t'">sizeof(UInt32)</xsl:when>
                      <xsl:when test="$original_type='u_int64_t'">sizeof(UInt64)</xsl:when>
                      <xsl:when test="$original_type='enum8_t'">sizeof(Byte)</xsl:when>
                      <xsl:when test="$original_type='enum16_t'">sizeof(UInt16)</xsl:when>
                      <xsl:when test="$original_type='enum32_t'">sizeof(UInt32)</xsl:when>
                      <xsl:when test="$original_type='enum64_t'">sizeof(UInt64)</xsl:when>
                      <xsl:when test="$original_type='ieee754_32_t'">sizeof(Single)</xsl:when>
                      <xsl:when test="$original_type='ieee754_64_t'">sizeof(Double)</xsl:when>
                      <xsl:when test="$original_type='struct'">(int)Nerv36.Data<xsl:value-of select="$real_data_domain_namespace"/>.<xsl:value-of select="real_data"/>._sizeof</xsl:when>
                      <xsl:otherwise>Nerv36.<xsl:value-of select="type"/>._sizeof</xsl:otherwise>
                    </xsl:choose>
                  </xsl:variable>
                  <xsl:if test="arraytype='VARIABLE'">+<xsl:value-of select="$member_size"/>*<xsl:value-of select="arraysize"/></xsl:if>
                </xsl:for-each>
              </xsl:variable>

              <xsl:variable name="struct_size">
                <xsl:for-each select="datafield/row">
                  <xsl:variable name="original_type">
                    <xsl:if test="type='alias'">
                      <xsl:call-template name="get_original_type">
                        <xsl:with-param name="data_domain_id" select="real_domain_id" />
                        <xsl:with-param name="data" select="real_data" />
                      </xsl:call-template>
                    </xsl:if>
                    <xsl:if test="type!='alias'">
                      <xsl:value-of select="type"/>
                    </xsl:if>
                  </xsl:variable>
                  <xsl:variable name="real_data_domain_namespace">
                    <xsl:call-template name="get_data_domain_namespace_dot">
                      <xsl:with-param name="data_domain_id" select="real_domain_id" />
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:variable name="member_size">
                    <xsl:choose>
                      <xsl:when test="$original_type='char_t'">sizeof(Byte)</xsl:when>
                      <xsl:when test="$original_type='u_scaled64_t'">sizeof(UInt64)</xsl:when>
                      <xsl:when test="$original_type='u_scaled32_t'">sizeof(UInt32)</xsl:when>
                      <xsl:when test="$original_type='u_scaled16_t'">sizeof(UInt16)</xsl:when>
                      <xsl:when test="$original_type='u_scaled8_t'">sizeof(Byte)</xsl:when>
                      <xsl:when test="$original_type='scaled64_t'">sizeof(Int64)</xsl:when>
                      <xsl:when test="$original_type='scaled32_t'">sizeof(Int32)</xsl:when>
                      <xsl:when test="$original_type='scaled16_t'">sizeof(Int16)</xsl:when>
                      <xsl:when test="$original_type='scaled8_t'">sizeof(SByte)</xsl:when>
                      <xsl:when test="$original_type='int8_t'">sizeof(SByte)</xsl:when>
                      <xsl:when test="$original_type='int16_t'">sizeof(Int16)</xsl:when>
                      <xsl:when test="$original_type='int32_t'">sizeof(Int32)</xsl:when>
                      <xsl:when test="$original_type='int64_t'">sizeof(Int64)</xsl:when>
                      <xsl:when test="$original_type='u_int8_t'">sizeof(Byte)</xsl:when>
                      <xsl:when test="$original_type='u_int16_t'">sizeof(UInt16)</xsl:when>
                      <xsl:when test="$original_type='u_int32_t'">sizeof(UInt32)</xsl:when>
                      <xsl:when test="$original_type='u_int64_t'">sizeof(UInt64)</xsl:when>
                      <xsl:when test="$original_type='enum8_t'">sizeof(Byte)</xsl:when>
                      <xsl:when test="$original_type='enum16_t'">sizeof(UInt16)</xsl:when>
                      <xsl:when test="$original_type='enum32_t'">sizeof(UInt32)</xsl:when>
                      <xsl:when test="$original_type='enum64_t'">sizeof(UInt64)</xsl:when>
                      <xsl:when test="$original_type='ieee754_32_t'">sizeof(Single)</xsl:when>
                      <xsl:when test="$original_type='ieee754_64_t'">sizeof(Double)</xsl:when>
                      <xsl:when test="$original_type='struct'">(int)Nerv36.Data<xsl:value-of select="$real_data_domain_namespace"/>.<xsl:value-of select="real_data"/>._sizeof</xsl:when>
                      <xsl:otherwise>Nerv36.<xsl:value-of select="type"/>._sizeof</xsl:otherwise>
                    </xsl:choose>
                  </xsl:variable>
                  <xsl:if test="arraytype='NO'">+<xsl:value-of select="$member_size"/></xsl:if>
                  <xsl:if test="arraytype='FIX'">+<xsl:value-of select="$member_size"/>*<xsl:value-of select="arraysize"/></xsl:if>
                  <xsl:if test="arraytype='VARIABLE'">+sizeof(UInt64)</xsl:if>
                </xsl:for-each>
              </xsl:variable>
    private Int64 __varray_offset; //
    public const Int32 _sizeof=sizeof(Int64)<xsl:value-of select="$struct_size"/>;

    public Int32 sizeofall()
    {
        return _sizeof + sizeof(UInt32)*3 + (Int32)dynamicmax();
    }

    public void init_dynamic_buffer(Int64 _size)
    {
        fixed( Int64* p = &amp;__varray_offset )
        {
            UInt32* struct_offset_ = (UInt32*)((Byte*)p + <xsl:value-of select="name"/>._sizeof + 0 * sizeof(UInt32));
            UInt32* dynamicmax_ = (UInt32*)((Byte*)p + <xsl:value-of select="name"/>._sizeof + 1 * sizeof(UInt32));
            UInt32* dynamicsize_ = (UInt32*)((Byte*)p + <xsl:value-of select="name"/>._sizeof + 2 * sizeof(UInt32));
            *struct_offset_ = <xsl:value-of select="name"/>._sizeof;
            *dynamicmax_ = (UInt32)_size;
            *dynamicsize_ = 0;
            init_varray_offset( <xsl:value-of select="name"/>._sizeof );
        }
    }

    public static ref <xsl:value-of select="name"/> malloc(Int32 _size = 65536)
    {
        // 할당크기는 구조체 기본크기 + 가변버퍼 할당 제어 64bit index + 가변버퍼총크기
        Int32 size = <xsl:value-of select="name"/>._sizeof + 3*sizeof(UInt32) +  _size;
        IntPtr buf_ = Marshal.AllocHGlobal(size);

        <xsl:value-of select="name"/>* new_ = (<xsl:value-of select="name"/>*)buf_;

        new_->init_dynamic_buffer(_size);

        return ref *new_;
    }
    public ref <xsl:value-of select="name"/> realloc(UInt32 _size = 0)
    {
        fixed (Int64* p = &amp;__varray_offset)
        {
            if ( _size &lt;= dynamicmax() ) _size = dynamicmax()*2;
            void* buf_ = (void*)Marshal.ReAllocHGlobal((IntPtr)p, (IntPtr)(<xsl:value-of select="name"/>._sizeof + 3*sizeof(UInt32) +  _size) );
            <xsl:value-of select="name"/>* new_ = (<xsl:value-of select="name"/>*)buf_;
            new_->dynamicmax() = _size;
            return ref *new_;
        }
    }

    public void free()
    {
        fixed (Int64* p = &amp;__varray_offset)
        {
            Marshal.FreeHGlobal((IntPtr)p);
        }
    }

    public UInt32 fit_size()
    {
        return <xsl:value-of select="name"/>._sizeof + 3*sizeof(UInt32) + dynamicsize();
    }

    public IntPtr memory()
    {
        fixed (Int64* p = &amp;__varray_offset)
        {
            return (IntPtr)((Byte*)p + __varray_offset - struct_offset());
        }
    }

    private ref UInt32 struct_offset()
    {
        fixed (Int64* p = &amp;__varray_offset)
        {
            return ref *(UInt32*)((Byte*)p + __varray_offset);
        }
    }
    private ref UInt32 dynamicmax()
    {
        fixed (Int64* p = &amp;__varray_offset )
        {
            UInt32* a = (UInt32*)((Byte*)p + __varray_offset + sizeof(UInt32));
            return ref *a;
        }
    }

    private ref UInt32 dynamicsize()
    {
        fixed (Int64* p = &amp;__varray_offset)
        {
            return ref *(UInt32*)((Byte*)p + __varray_offset + 2 * sizeof(UInt32));
        }
    }

    private Byte* dynamicbuffer()
    {
        fixed (Int64* p = &amp;__varray_offset)
        {
            return ((Byte*)p + __varray_offset + 3 * sizeof(UInt32));
        }
    }

    public void init_varray_offset(Int64 _varray_offset)
    {
        __varray_offset = _varray_offset;

        fixed( Int64* p = &amp;__varray_offset ) {
        <!-- // Narray 나 Fix 의 경우 init_varray_offset를 모두 수행해준다.
        // 내부에 가변버퍼를 가지는 멤버가 있을수 있기 때문이다. -->
            <xsl:for-each select="datafield/row[type='alias']">
              <xsl:variable name="original_type">
                <xsl:if test="type='alias'">
                  <xsl:call-template name="get_original_type">
                    <xsl:with-param name="data_domain_id" select="real_domain_id" />
                    <xsl:with-param name="data" select="real_data" />
                  </xsl:call-template>
                </xsl:if>
                <xsl:if test="type!='alias'">
                  <xsl:value-of select="type"/>
                </xsl:if>
              </xsl:variable>
              <xsl:if test="arraytype='NO'">
                <xsl:if test="($original_type='struct')">
            fixed(Byte* buf = <xsl:value-of select="member_name"/>_buffer) {
                Int64 member_varray_offset = __varray_offset;
                member_varray_offset -= (Int64)( buf - (Byte*)p );
                <xsl:value-of select="member_name"/>().init_varray_offset( member_varray_offset );
            }
                </xsl:if>
              </xsl:if>
              <xsl:if test="arraytype='FIX'">
                <xsl:variable name="real_data_domain_namespace">
                  <xsl:call-template name="get_data_domain_namespace_dot">
                    <xsl:with-param name="data_domain_id" select="real_domain_id" />
                  </xsl:call-template>
                </xsl:variable>
            fixed( Byte* buf = <xsl:value-of select="member_name"/>_buffer) {
                for(int i=0;i&lt;<xsl:value-of select="arraysize"/>;i++) {
                    Int64 member_varray_offset = __varray_offset;
                    member_varray_offset -= (Int64)(buf + Nerv36.Data<xsl:value-of select="$real_data_domain_namespace"/>.<xsl:value-of select="real_data"/>._sizeof*i - (Byte*)p);
                    <xsl:value-of select="member_name"/>(i).init_varray_offset( member_varray_offset );
                }
            }
              </xsl:if>
            </xsl:for-each>
        }
    }
              <xsl:for-each select="datafield/row">
                <xsl:variable name="description_aligned">
                  <xsl:call-template name="replace">
                    <xsl:with-param name="text" select="description" />
                    <xsl:with-param name="replace" select="'&#10;'" />
                    <xsl:with-param name="by" select="'&#10;    // '" />
                  </xsl:call-template>
                </xsl:variable>
                <xsl:variable name="original_type">
                  <xsl:if test="type='alias'">
                    <xsl:call-template name="get_original_type">
                      <xsl:with-param name="data_domain_id" select="real_domain_id" />
                      <xsl:with-param name="data" select="real_data" />
                    </xsl:call-template>
                  </xsl:if>
                  <xsl:if test="type!='alias'">
                    <xsl:value-of select="type"/>
                  </xsl:if>
                </xsl:variable>
                <xsl:variable name="real_data_domain_namespace">
                  <xsl:call-template name="get_data_domain_namespace_dot">
                    <xsl:with-param name="data_domain_id" select="real_domain_id" />
                  </xsl:call-template>
                </xsl:variable>
                <xsl:variable name="member_size">
                  <xsl:choose>
                    <xsl:when test="$original_type='char_t'">sizeof(Byte)</xsl:when>
                    <xsl:when test="$original_type='u_scaled64_t'">sizeof(UInt64)</xsl:when>
                    <xsl:when test="$original_type='u_scaled32_t'">sizeof(UInt32)</xsl:when>
                    <xsl:when test="$original_type='u_scaled16_t'">sizeof(UInt16)</xsl:when>
                    <xsl:when test="$original_type='u_scaled8_t'">sizeof(Byte)</xsl:when>
                    <xsl:when test="$original_type='scaled64_t'">sizeof(Int64)</xsl:when>
                    <xsl:when test="$original_type='scaled32_t'">sizeof(Int32)</xsl:when>
                    <xsl:when test="$original_type='scaled16_t'">sizeof(Int16)</xsl:when>
                    <xsl:when test="$original_type='scaled8_t'">sizeof(SByte)</xsl:when>
                    <xsl:when test="$original_type='int8_t'">sizeof(SByte)</xsl:when>
                    <xsl:when test="$original_type='int16_t'">sizeof(Int16)</xsl:when>
                    <xsl:when test="$original_type='int32_t'">sizeof(Int32)</xsl:when>
                    <xsl:when test="$original_type='int64_t'">sizeof(Int64)</xsl:when>
                    <xsl:when test="$original_type='u_int8_t'">sizeof(Byte)</xsl:when>
                    <xsl:when test="$original_type='u_int16_t'">sizeof(UInt16)</xsl:when>
                    <xsl:when test="$original_type='u_int32_t'">sizeof(UInt32)</xsl:when>
                    <xsl:when test="$original_type='u_int64_t'">sizeof(UInt64)</xsl:when>
                    <xsl:when test="$original_type='enum8_t'">sizeof(Byte)</xsl:when>
                    <xsl:when test="$original_type='enum16_t'">sizeof(UInt16)</xsl:when>
                    <xsl:when test="$original_type='enum32_t'">sizeof(UInt32)</xsl:when>
                    <xsl:when test="$original_type='enum64_t'">sizeof(UInt64)</xsl:when>
                    <xsl:when test="$original_type='ieee754_32_t'">sizeof(Single)</xsl:when>
                    <xsl:when test="$original_type='ieee754_64_t'">sizeof(Double)</xsl:when>
                    <xsl:when test="$original_type='struct'">(int)Nerv36.Data<xsl:value-of select="$real_data_domain_namespace"/>.<xsl:value-of select="real_data"/>._sizeof</xsl:when>
                    <xsl:otherwise>Nerv36.<xsl:value-of select="type"/>._sizeof</xsl:otherwise>
                  </xsl:choose>
                </xsl:variable>
                <xsl:variable name="member_type">
                  <xsl:choose>
                    <xsl:when test="type='char_t'">Byte</xsl:when>
                    <xsl:when test="type='u_scaled64_t'">_<xsl:value-of select="member_name"/></xsl:when>
                    <xsl:when test="type='u_scaled32_t'">_<xsl:value-of select="member_name"/></xsl:when>
                    <xsl:when test="type='u_scaled16_t'">_<xsl:value-of select="member_name"/></xsl:when>
                    <xsl:when test="type='u_scaled8_t'">_<xsl:value-of select="member_name"/></xsl:when>
                    <xsl:when test="type='scaled64_t'">_<xsl:value-of select="member_name"/></xsl:when>
                    <xsl:when test="type='scaled32_t'">_<xsl:value-of select="member_name"/></xsl:when>
                    <xsl:when test="type='scaled16_t'">_<xsl:value-of select="member_name"/></xsl:when>
                    <xsl:when test="type='scaled8_t'">_<xsl:value-of select="member_name"/></xsl:when>
                    <xsl:when test="type='alias'">Nerv36.Data<xsl:value-of select="$real_data_domain_namespace"/>.<xsl:value-of select="real_data"/></xsl:when>
                    <xsl:when test="type='u_int8_t'">Byte</xsl:when>
                    <xsl:when test="type='u_int16_t'">UInt16</xsl:when>
                    <xsl:when test="type='u_int32_t'">UInt32</xsl:when>
                    <xsl:when test="type='u_int64_t'">UInt64</xsl:when>
                    <xsl:when test="type='int8_t'">SByte</xsl:when>
                    <xsl:when test="type='int16_t'">Int16</xsl:when>
                    <xsl:when test="type='int32_t'">Int32</xsl:when>
                    <xsl:when test="type='int64_t'">Int64</xsl:when>
                    <xsl:when test="type='ieee754_32_t'">Single</xsl:when>
                    <xsl:when test="type='ieee754_64_t'">Double</xsl:when>
                  </xsl:choose>
                </xsl:variable>
<xsl:text>
    </xsl:text>// <xsl:value-of select="$description_aligned"/><xsl:text/>
                <xsl:if test="type='scaled64_t' or type='scaled32_t' or type='scaled16_t' or type='scaled8_t' or type='u_scaled64_t' or type='u_scaled32_t' or type='u_scaled16_t' or type='u_scaled8_t'">
<xsl:text>
    </xsl:text>public unsafe struct _<xsl:value-of select="member_name"/>
    {
        public const double min = <xsl:value-of select="scmin"/>;
        public const double max = <xsl:value-of select="scmax"/>;

                <xsl:if test="type='scaled64_t'">
        public Int64 value;
                </xsl:if>
                <xsl:if test="type='scaled32_t'">
        public Int32 value;
                </xsl:if>
                <xsl:if test="type='scaled16_t'">
        public Int16 value;
                </xsl:if>
                <xsl:if test="type='scaled8_t'">
        public SByte value;
                </xsl:if>
                <xsl:if test="type='u_scaled64_t'">
        public UInt64 value;
                </xsl:if>
                <xsl:if test="type='u_scaled32_t'">
        public UInt32 value;
                </xsl:if>
                <xsl:if test="type='u_scaled16_t'">
        public UInt16 value;
                </xsl:if>
                <xsl:if test="type='u_scaled8_t'">
        public Byte value;
                </xsl:if>
    }
                </xsl:if>
                <xsl:if test="arraytype='NO'">
                  <xsl:if test="($original_type!='struct')">
<xsl:text>
    </xsl:text>public <xsl:value-of select="$member_type"/><xsl:text> </xsl:text><xsl:value-of select="member_name"/><xsl:if test="not(bitsize = '0')">; // :<xsl:value-of select="bitsize"/></xsl:if>;<xsl:text/>
                  </xsl:if>
                  <xsl:if test="($original_type='struct')">
<xsl:text>
    </xsl:text>fixed Byte <xsl:value-of select="member_name"/>_buffer[(int)<xsl:value-of select="$member_type"/>._sizeof];<xsl:text/>
    public ref <xsl:value-of select="$member_type"/><xsl:text> </xsl:text><xsl:value-of select="member_name"/>()
    {
        fixed(Byte* buffer_ = <xsl:value-of select="member_name"/>_buffer)
        {
            <xsl:value-of select="$member_type"/>* ptr = (<xsl:value-of select="$member_type"/>*)(buffer_);
            return ref *ptr;
        }
    }<xsl:text/>
                  </xsl:if>
                  <xsl:if test="(type='alias') and (bitsize = '0')">
                  </xsl:if>
                </xsl:if>
                <xsl:if test="arraytype='FIX'">
<xsl:text>
    </xsl:text>

                  <xsl:if test="($original_type!='struct')">
    // [MarshalAs(UnmanagedType.LPArray, SizeConst = <xsl:value-of select="arraysize"/>)]
    public fixed <xsl:value-of select="$member_type"/><xsl:text> </xsl:text><xsl:value-of select="member_name"/>[<xsl:value-of select="arraysize"/>];<xsl:text/>
                  </xsl:if>
                  <xsl:if test="($original_type='struct')">
    fixed Byte <xsl:value-of select="member_name"/>_buffer[(int)<xsl:value-of select="$member_type"/>._sizeof*<xsl:value-of select="arraysize"/>];<xsl:text/>
    public ref <xsl:value-of select="$member_type"/><xsl:text> </xsl:text><xsl:value-of select="member_name"/>(int _index)
    {
        fixed(Byte* buffer_ = <xsl:value-of select="member_name"/>_buffer)
        {
            <xsl:value-of select="$member_type"/>* ptr = (<xsl:value-of select="$member_type"/>*)(buffer_);
            return ref ptr[_index];
        }
    }<xsl:text/>
                  </xsl:if>

                </xsl:if>
                <xsl:if test="arraytype='VARIABLE'">
<xsl:text>
    </xsl:text>public Variable_array_field_dctrl <xsl:value-of select="member_name"/>_dctrl;
    public <xsl:value-of select="$member_type"/>*<xsl:text> </xsl:text><xsl:value-of select="member_name"/>()
    {
        <xsl:value-of select="$member_type"/>* ptr = (<xsl:value-of select="$member_type"/>*)(dynamicbuffer() + <xsl:value-of select="member_name"/>_dctrl.dindex);
        return ptr;
    }<xsl:text/>
<xsl:text>
    public </xsl:text>UInt64 <xsl:value-of select="member_name"/>_alloc(UInt32 _size)
    {
        UInt64 need_dynamics_size;
        ref UInt32 dynamicmax_ = ref dynamicmax();
        ref UInt32 dynamicsize_ = ref dynamicsize();
        <xsl:value-of select="member_name"/>_dctrl.dindex = dynamicsize_;
        <xsl:if test="$original_type='struct'">UInt32 byte_size = _size*<xsl:value-of select="$member_type"/>._sizeof;</xsl:if>
        <xsl:if test="$original_type!='struct'">UInt32 byte_size = _size*sizeof(<xsl:value-of select="$member_type"/>);</xsl:if>

        if(dynamicmax_ &lt; dynamicsize_ + byte_size) {
            need_dynamics_size = dynamicsize_ + byte_size;
        }
        else {
            <xsl:value-of select="member_name"/>_dctrl.dsize = byte_size;
            dynamicsize_ += <xsl:value-of select="member_name"/>_dctrl.dsize;
            <xsl:if test="type='alias'">
            <xsl:value-of select="$member_type"/>* member_ = <xsl:value-of select="member_name"/>();
            for(int i=0;i&lt;_size;i++) {
                Int64 member_varray_offset = __varray_offset;
                fixed(Int64* p = &amp;__varray_offset)
                {
                    member_varray_offset -= (Int64)( (Byte*)(member_+i) - (Byte*)p );
                }
                member_[i].init_varray_offset( member_varray_offset );
            }
            </xsl:if>
            need_dynamics_size=0;
        }
        return need_dynamics_size;
    }<xsl:text/>
<xsl:text>
    public </xsl:text>UInt32 <xsl:value-of select="member_name"/>_size()
    {
        return <xsl:value-of select="member_name"/>_dctrl.dsize / <xsl:value-of select="$member_size"/>;
    }
                </xsl:if>
<xsl:text>
</xsl:text>
              </xsl:for-each>
            </xsl:if>

    public void set(int val)
    {
            <xsl:if test="type='alias'">
        ori().set(val);<xsl:text/>
            </xsl:if>
            <xsl:if test="type!='struct' and type!='alias'">
        value_buffer = (<xsl:value-of select="$value_type" />)val;<xsl:text/>
            </xsl:if>
            <xsl:if test="type='struct'">
              <xsl:for-each select="datafield/row">
                <xsl:variable name="real_data_domain_namespace">
                  <xsl:call-template name="get_data_domain_namespace_dot">
                    <xsl:with-param name="data_domain_id" select="real_domain_id" />
                  </xsl:call-template>
                </xsl:variable>
                <xsl:variable name="member_type">
                  <xsl:choose>
                    <xsl:when test="type='char_t'">Byte</xsl:when>
                    <xsl:when test="type='u_scaled64_t'">UInt64</xsl:when>
                    <xsl:when test="type='u_scaled32_t'">UInt32</xsl:when>
                    <xsl:when test="type='u_scaled16_t'">UInt16</xsl:when>
                    <xsl:when test="type='u_scaled8_t'">Byte</xsl:when>
                    <xsl:when test="type='scaled64_t'">Int64</xsl:when>
                    <xsl:when test="type='scaled32_t'">Int32</xsl:when>
                    <xsl:when test="type='scaled16_t'">Int16</xsl:when>
                    <xsl:when test="type='scaled8_t'">SByte</xsl:when>
                    <xsl:when test="type='alias'">Nerv36.Data<xsl:value-of select="$real_data_domain_namespace"/>.<xsl:value-of select="real_data"/></xsl:when>
                    <xsl:when test="type='u_int8_t'">Byte</xsl:when>
                    <xsl:when test="type='u_int16_t'">UInt16</xsl:when>
                    <xsl:when test="type='u_int32_t'">UInt32</xsl:when>
                    <xsl:when test="type='u_int64_t'">UInt64</xsl:when>
                    <xsl:when test="type='int8_t'">SByte</xsl:when>
                    <xsl:when test="type='int16_t'">Int16</xsl:when>
                    <xsl:when test="type='int32_t'">Int32</xsl:when>
                    <xsl:when test="type='int64_t'">Int64</xsl:when>
                    <xsl:when test="type='ieee754_32_t'">Single</xsl:when>
                    <xsl:when test="type='ieee754_64_t'">Double</xsl:when>
                  </xsl:choose>
                </xsl:variable>
                <xsl:variable name="original_type">
                  <xsl:if test="type='alias'">
                    <xsl:call-template name="get_original_type">
                      <xsl:with-param name="data_domain_id" select="real_domain_id" />
                      <xsl:with-param name="data" select="real_data" />
                    </xsl:call-template>
                  </xsl:if>
                  <xsl:if test="type!='alias'">
                    <xsl:value-of select="type"/>
                  </xsl:if>
                </xsl:variable>
                <xsl:choose>
                  <xsl:when test="arraytype='NO'">
                    <xsl:choose>
                      <xsl:when test="$original_type='struct'">
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />().set(val);<xsl:text/>
                      </xsl:when>
                      <xsl:when test="type='scaled8_t' or type='scaled16_t' or type='scaled32_t' or type='scaled64_t' or type='u_scaled8_t' or type='u_scaled16_t' or type='u_scaled32_t' or type='u_scaled64_t'">
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />.value = (<xsl:value-of select="$member_type" />)val;<xsl:text/>
                      </xsl:when>
                      <xsl:otherwise>
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" /> = (<xsl:value-of select="$member_type" />)val;<xsl:text/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:when>
                  <xsl:when test="arraytype='FIX'">
                    <xsl:choose>
                      <xsl:when test="$original_type='struct'">
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />(0).set(val);<xsl:text/>
                      </xsl:when>
                      <xsl:when test="type='scaled8_t' or type='scaled16_t' or type='scaled32_t' or type='scaled64_t' or type='u_scaled8_t' or type='u_scaled16_t' or type='u_scaled32_t' or type='u_scaled64_t'">
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />[0].value = (<xsl:value-of select="$member_type" />)val;<xsl:text/>
                      </xsl:when>
                      <xsl:otherwise>
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />[0] = (<xsl:value-of select="$member_type" />)val;<xsl:text/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:when>
                  <xsl:when test="arraytype='VARIABLE'">
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />_alloc(1);<xsl:text/>
                    <xsl:choose>
                      <xsl:when test="type='alias'">
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />()[0].set(val);<xsl:text/>
                      </xsl:when>
                      <xsl:when test="type='scaled8_t' or type='scaled16_t' or type='scaled32_t' or type='scaled64_t' or type='u_scaled8_t' or type='u_scaled16_t' or type='u_scaled32_t' or type='u_scaled64_t'">
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />()[0].value_buffer = (<xsl:value-of select="$member_type" />)val;<xsl:text/>
                      </xsl:when>
                      <xsl:otherwise>
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />()[0] = (<xsl:value-of select="$member_type" />)val;<xsl:text/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:when>
                </xsl:choose>
              </xsl:for-each>
            </xsl:if>
    }

    public void print(string var)
    {
            <xsl:if test="type='alias'">
        ori().print(var);<xsl:text/>
            </xsl:if>
            <xsl:if test="type!='struct' and type!='alias'">
        Output.Log( string.Format( var+" = {0}" , value_buffer));<xsl:text/>
            </xsl:if>
            <xsl:if test="type='struct'">
        Output.Log( string.Format( var+"._sizeof()={0}" , _sizeof));<xsl:text/>
        Output.Log( string.Format( var+".fit_size()={0}" , fit_size()));<xsl:text/>
              <xsl:for-each select="datafield/row">
                <xsl:variable name="original_type">
                  <xsl:if test="type='alias'">
                    <xsl:call-template name="get_original_type">
                      <xsl:with-param name="data_domain_id" select="real_domain_id" />
                      <xsl:with-param name="data" select="real_data" />
                    </xsl:call-template>
                  </xsl:if>
                  <xsl:if test="type!='alias'">
                    <xsl:value-of select="type"/>
                  </xsl:if>
                </xsl:variable>
                <xsl:choose>
                  <xsl:when test="arraytype='NO'">
                    <xsl:choose>
                      <xsl:when test="$original_type='struct'">
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />().print(var+".<xsl:value-of select="member_name" />");<xsl:text/>
                      </xsl:when>
                      <xsl:when test="type='scaled8_t' or type='scaled16_t' or type='scaled32_t' or type='scaled64_t' or type='u_scaled8_t' or type='u_scaled16_t' or type='u_scaled32_t' or type='u_scaled64_t'">
        Output.Log( string.Format( var+".<xsl:value-of select="member_name"/>= {0}" , <xsl:value-of select="member_name" />.value));<xsl:text/>
                      </xsl:when>
                      <xsl:otherwise>
        Output.Log( string.Format( var+".<xsl:value-of select="member_name"/>= {0}" , <xsl:value-of select="member_name" />));<xsl:text/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:when>
                  <xsl:when test="arraytype='FIX'">
                    <xsl:choose>
                      <xsl:when test="$original_type='struct'">
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />(0).print(var+".<xsl:value-of select="member_name" />(0)");<xsl:text/>
                      </xsl:when>
                      <xsl:when test="type='scaled8_t' or type='scaled16_t' or type='scaled32_t' or type='scaled64_t' or type='u_scaled8_t' or type='u_scaled16_t' or type='u_scaled32_t' or type='u_scaled64_t'">
        Output.Log( string.Format( var+".<xsl:value-of select="member_name"/>[0] = {0}" , <xsl:value-of select="member_name" />[0].value));<xsl:text/>
                      </xsl:when>
                      <xsl:otherwise>
        Output.Log( string.Format( var+".<xsl:value-of select="member_name"/>[0] = {0}" , <xsl:value-of select="member_name" />[0]));<xsl:text/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:when>
                  <xsl:when test="arraytype='VARIABLE'">
                    <xsl:choose>
                      <xsl:when test="type='alias'">
<xsl:text>
        </xsl:text><xsl:value-of select="member_name" />()[0].print(var+".<xsl:value-of select="member_name" />[0]");<xsl:text/>
                      </xsl:when>
                      <xsl:when test="type='scaled8_t' or type='scaled16_t' or type='scaled32_t' or type='scaled64_t' or type='u_scaled8_t' or type='u_scaled16_t' or type='u_scaled32_t' or type='u_scaled64_t'">
        Output.Log( string.Format( var+".<xsl:value-of select="member_name"/>[0] = {0}" , <xsl:value-of select="member_name" />()[0].value_buffer));<xsl:text/>
                      </xsl:when>
                      <xsl:otherwise>
        Output.Log( string.Format( var+".<xsl:value-of select="member_name"/>[0] = {0}" , <xsl:value-of select="member_name" />()[0]));<xsl:text/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </xsl:when>
                </xsl:choose>
              </xsl:for-each>
            </xsl:if>
    }
}
          </xsl:if>

//inline UInt32 nerv_sizeof( <xsl:value-of select="name"/> obj)
//{
//    return obj.get_sizeof();
//}
// typedef template_history_elem&lt;<xsl:value-of select="name"/>&gt; <xsl:value-of select="name"/>_history_elem;
// typedef template_history&lt;<xsl:value-of select="name"/>&gt; <xsl:value-of select="name"/>_history;


          <xsl:if test="type!='struct'">
// typedef template_history_elem&lt;<xsl:value-of select="name"/>&gt; <xsl:value-of select="name"/>_history_elem;
// typedef template_history&lt;<xsl:value-of select="name"/>&gt; <xsl:value-of select="name"/>_history;
          </xsl:if>

} } /* <xsl:value-of select="$close_namespace_path" /> */
        </xsl:if>
      </xsl:for-each>
    </xsl:if>
  </xsl:for-each>
</xsl:template>


</xsl:stylesheet>
