#pragma once

#include <condition_variable>
#include <mutex>
#include <thread>

#include <Waregram/_debug.hpp>

#ifdef _MSC_VER
#include <Windows.h>
#endif

static void wg_delete_join_thread(std::thread*& thread_)
{
    std::thread* local_ = thread_;
    thread_ = NULL;
    if(local_->joinable()) local_->join();
    TRACE("joined");
    delete local_;
}

struct WG_event {
    std::condition_variable condition;
    std::mutex              mutex;
};

static void wg_wait_condition(WG_event& _event_)
{
    std::unique_lock lock(_event_.mutex);
    _event_.condition.wait(lock);
}

static std::cv_status wg_wait_condition_for(WG_event& _event_, double limit)
{
    auto nano_itvl = std::chrono::nanoseconds{int64_t(limit * 1000000000)};

    std::unique_lock lock(_event_.mutex);
    std::cv_status cvs = _event_.condition.wait_for(lock, nano_itvl);

#ifdef _MSC_VER
    if( cvs == std::cv_status::no_timeout ) {
        if( GetLastError() == ERROR_TIMEOUT ) {
            cvs = std::cv_status::timeout;
            // MessageBox(0,"msc bug", "64nerv36", MB_OK);
        }
    }
#endif

    return cvs;
}

typedef void thread_entry_t;