<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>
<xsl:template match="/">

#include &lt;boost/python.hpp&gt;
#include &lt;Nerv36/nerv_utility.hh&gt;



// data export
<xsl:for-each select="/nerv_icde/datadomain/row">
#include &lt;Nerv36/Data<xsl:value-of select="path"/>/<xsl:value-of select="name"/>_export.h&gt;
</xsl:for-each>

// proxy export
<xsl:for-each select="/nerv_icde/component/proxy_list/row">
#include "Nerv/Proxy/<xsl:value-of select="class_name"/>/<xsl:value-of select="class_name"/>_export.h"
</xsl:for-each>

// stub export
<xsl:for-each select="/nerv_icde/component/stub_list/row">
#include "Nerv/Stub/<xsl:value-of select="class_name"/>/<xsl:value-of select="class_name"/>_export.h"
</xsl:for-each>



#include &lt;Waregram/_inet.hpp&gt;

#include &lt;jsoncpp/json.h&gt;

// #include &lt;Winsock2.h&gt;
// #include &lt;sys/types.h&gt;
// #include &lt;sys/socket.h&gt;
// #include &lt;fcntl.h&gt;
// #include &lt;netinet/in.h&gt;

Socket s = INVALID_SOCKET;



void close_log_server()
{
    if(s!=INVALID_SOCKET) {
        closesocket(s);
        s = INVALID_SOCKET;
        printf("close log Client.\n");
    }
}

bool connect_log_server(const char* _ip_, unsigned short _port)
{
    if(s==INVALID_SOCKET) {
        s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

        sockaddr_in server_addr;
        memset(&amp;server_addr, 0, sizeof(sockaddr_in));
        server_addr.sin_family      = (Sa_family)AF_INET;
        server_addr.sin_addr.s_addr = Nerv36::inet_addr(_ip_);
        server_addr.sin_port        = htons(_port);
        int r = connect(s, (sockaddr*)&amp;server_addr, (Socklen)sizeof(sockaddr_in));
        if(r) {
            closesocket(s);
            s = INVALID_SOCKET;
            printf("failed to connect to Log Server.\n");
        }
        else {
            printf("connected Client to Log Server.\n");
            int nodelay = 1;
            setsockopt(s, IPPROTO_TCP, TCP_NODELAY,(char*)&amp;nodelay, 4);
        }
    }
    return s!=INVALID_SOCKET;
}

bool send_log_server(const char* _json_, size_t _size)
{
    bool b_ret = false;
    char* buffer_ = new char[_size+1];
    memcpy(buffer_, _json_, _size+1);
    buffer_[_size] = '\r';
    buffer_[_size+1] = '\n';

    printf("send\n");

    // int slen = (int)send(s,buffer_,_size+1,MSG_WAITALL);
    int slen = (int)send(s,buffer_,_size+1,0);

    if(slen==_size+1) {
        b_ret = true;
    }
    else {
        close_log_server();
    }
    delete [] buffer_;
    return b_ret;
}

bool send_log_server(const Json::Value&amp; _json)
{
    bool b_ret = false;
    if(s!=INVALID_SOCKET) {
        // Json::StyledWriter w;
        Json::FastWriter w;
        std::string str = w.write(_json);
        b_ret = send_log_server(str.c_str(),str.size());
    }
    return b_ret;
}



namespace py = boost::python;

void nerv_init()
{
    ::Nerv36::initialize();
}

void nerv_init_nonthread()
{
    ::Nerv36::initialize(0, 16, 1);
}

void nerv_finalize()
{
    ::Nerv36::finalize();
}

bool nerv_dispatch_msg()
{
    bool ret_b;
    Py_BEGIN_ALLOW_THREADS
    ret_b = ::Nerv36::dispatch_message();
    Py_END_ALLOW_THREADS
    return ret_b;
}

bool nerv_dispatch_msg_timeout(double _timeout)
{
    bool ret_b;
    Py_BEGIN_ALLOW_THREADS
    ret_b = ::Nerv36::dispatch_message(_timeout);
    Py_END_ALLOW_THREADS
    return ret_b;
}

::Nerv36::IP nerv_inet_addr(const char* _str_ip)
{
    return ::Nerv36::inet_addr(_str_ip);
}

// 도메인의 c_standard_port 얻는 함수로 제공
<xsl:for-each select="/nerv_icde/methoddomain/row">
int get_<xsl:value-of select="name"/>_port()
{
    <xsl:variable name="method_domain_namespace">
        <xsl:call-template name="get_method_domain_namespace">
            <xsl:with-param name="method_domain_id" select="id" />
        </xsl:call-template>
    </xsl:variable>
    return Nerv36::Method<xsl:value-of select="$method_domain_namespace"/>::c_standard_port;
}
</xsl:for-each>


void export_py_nerv()
{
    py::class_&lt;::Nerv36::Address&gt;("Address")
        .def_readwrite("ip", &amp;::Nerv36::Address::ip)
        .def_readwrite("port", &amp;::Nerv36::Address::port)
        ;

    py::class_&lt;::Nerv36::Msg_num_ref&gt;("Msg_num_ref")
        .def_readwrite("ref", &amp;::Nerv36::Msg_num_ref::ref)
        ;

    py::def("close_log_server", &amp;close_log_server);
    py::def("connect_log_server", &amp;connect_log_server);

    py::def("initialize", &amp;nerv_init);
    py::def("initialize_nonthread", &amp;nerv_init_nonthread);
    py::def("finalize", &amp;nerv_finalize);
    py::def("dispatch_msg", &amp;nerv_dispatch_msg);
    py::def("dispatch_msg_timeout", &amp;nerv_dispatch_msg_timeout);
    py::def("inet_addr", &amp;nerv_inet_addr);
    py::def("inet_ntoa", &amp;Nerv36::inet_ntoa);
    py::def("get_err_str", &amp;Nerv36::get_err_str);

    // export c_standard_port
    <xsl:for-each select="/nerv_icde/methoddomain/row">
    py::def("get_<xsl:value-of select="name"/>_port", &amp;get_<xsl:value-of select="name"/>_port);
    </xsl:for-each>

    // export data
    <xsl:for-each select="/nerv_icde/datadomain/row">
        <xsl:variable name="data_domain_namespace">
            <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="id" />
            </xsl:call-template>
        </xsl:variable>
    Nerv36::Data<xsl:value-of select="$data_domain_namespace"/>::export_python_data();
    </xsl:for-each>

    // export proxy
    <xsl:for-each select="/nerv_icde/component/proxy_list/row">
    export_proxy_<xsl:value-of select="class_name"/>();
    </xsl:for-each>

    // export stub
    <xsl:for-each select="/nerv_icde/component/stub_list/row">
    export_stub_<xsl:value-of select="class_name"/>();
    </xsl:for-each>
}

BOOST_PYTHON_MODULE(py_nerv36)
{
    export_py_nerv();
}

<!-- int main()
{
    export_py_nerv();
} -->

</xsl:template>


</xsl:stylesheet>
