<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../xsl_util.xsl"/>
<xsl:output method="text" encoding="utf-8" indent="yes" />
<xsl:template match="/">
<xsl:variable name="component_name">
  <xsl:call-template name="replace">
    <xsl:with-param name="text" select="nerv_icde/component/name" />
    <xsl:with-param name="replace" select="' '" />
    <xsl:with-param name="by" select="'_'" />
  </xsl:call-template>
</xsl:variable>
/*******************************************************************************
Component Name: <xsl:value-of select="$component_name"/>
<xsl:value-of select="$component_name"/>_nerv_component 를 컴포넌트 클래스라고한다.
일반적으로 컴포넌트 이름을 stub객체 이름과 동일시 하기 때문에
클래스 이름 충돌을 회피하기 위하여
컴포넌트 클래스는 _nerv_component 접미사를 붙여서 작명되었다.

Nerv36::initialize() / finalize() 라이브러리 사이클 내에서만
미들웨어의 객체는 존재하여야 한다.

때문에 컴포넌트 클래스를 힙에서 할당(new연산자)하고
미들웨어 객체의 라이프사이클이
컴포넌트 클래스 라이프사이클 내에서만 존재하도록한다.

미들웨어 객체는 stub_/proxy_ 접두사가 붙도록 작명되고,
start시에 주어지는 "name"은 고유명사로서 활용한다.

    // 지역변수로 선언하면 구조체 크기가 큰경우 스택오버플로우가 발생하기 때문에
    // new / delete 힙 할당해제를 이용하는 안전 실행 코드가 자동 생성된다.
    // 만약 구조체 크기가 작다면 지역변수로 선언 가능하고 빠르다.
    {
        Nerv36::Data.Domain.Struct data;
        Int32 err = nerv_object.method.transaction(&amp;data);
    }

*******************************************************************************/

#include "<xsl:value-of select="$component_name"/>_nerv_component.hh"
#include "ros/ros.h"
#include &lt;Nerv36/nerv_env.hh&gt;

#include &lt;thread&gt;

using namespace std;
using namespace this_thread;
using namespace chrono;

// #include &lt;Waregram/utf8.h&gt;


int main(int argc, char* argv[])
{
    // wg_code_page cp_old = begin_utf8();
    const char* component_name = Nerv36::env("component_name","<xsl:value-of select="$component_name"/>");
    ros::init(argc, argv, component_name, ros::init_options::AnonymousName);
    Nerv36::initialize();

    <xsl:value-of select="$component_name"/>_nerv_component* nerv_component_;
    nerv_component_ = new <xsl:value-of select="$component_name"/>_nerv_component();
    // new 생성 직후에는 IDLE_ 상태이다.

    nerv_component_-&gt;ready();
    // ready 직후에는 STANDBY_ 상태이다.

    nerv_component_-&gt;start();
    // start 직후에는 RUN_ 상태이다.

    // 반복 순환문으로 동작하는 상태기계를 실행시킨다.
    // nerv_component_-&gt;excute_state_machine();
    // ros의 spin을 돌려야 하므로 별도의 thread에서 동작시킨다.
    thread state_machine(&amp;<xsl:value-of select="$component_name"/>_nerv_component::excute_state_machine, nerv_component_);

    ros::spin();

    nerv_component_-&gt;shutdown();
    state_machine.join();

    // end_utf8(cp_old);

    Nerv36::finalize();

    return 0;
}


</xsl:template>
</xsl:stylesheet>
